      module fakegrape
!****************************************************************
! This module is an addon component of VINE, copyright by       *
! the authors, as listed below. Please see the COPYING and      *
! LICENSE files in this directory for additional information.   *
! This notification is present as an attempt to follow the      *
! recommendations for licensing published on the FSF page.      * 
!****************************************************************
!                                                               *
! This module is an attempt to make an interface that replaces  *
! the grape library of routines, for those who want to do       *
! tests of the code that way and do not have actual hardware.   *
!                                                               *
! Currently, only grape5, grape6 and 6a are supported in        *
! this module.                                                  *
!                                                               *
! Note also that support is a relative term. There is enough    *
! support here to run the grapeN.F modules that VINE uses,      *
! in the way that VINE uses them. This is not a full            *
! reproduction of the entire suite of routines that the         *
! grape5/6/6a implements.                                       *
!                                                               *
! A warning for anyone who attempts to compile this module:     *
! The Fortran90/95 standard accepts routine names up to 31      *
! characters. Some compilers allow more than this, some don't.  *
! If your compiler doesn't allow longer names, then this module *
! will not compile as is. You'll have to replace the names      *
! with something shorter.                                       *
!                                                               *
! Module by Andy Nelson                                         *
!                                                               *
!****************************************************************
      use treevars
      use constants

      use wait

      implicit none

      private
!
! Public routines:
!

#if GRAPE==5 || GRAPE==565
      public g5_openMC , g5_closeMC , g5_set_rangeMC
      public g5_open   , g5_close   , g5_set_range
      public g5_set_nMC, g5_set_xjMC, g5_set_mjMC
      public g5_set_n  , g5_set_xmj
      public g5_set_eps_to_allMC,g5_set_eps_to_all
      public g5_get_firstcluster
      public g5_get_number_of_boards
      public g5_get_number_of_pipelines_per_board
      public g5_calculate_force_on_x
#endif

#if GRAPE==6 || GRAPE==65 || GRAPE==66
      public g6_open,g6_close,g6_npipes
      public g6_set_nip,g6_set_njp
      public g6_set_xunit,g6_set_tunit,g6_set_ti
      public g6_get_force
      public g6_set_i_particle
      public g6_set_j_particle
      public g6_set_i_particle_scales_from_real_value

#if GRAPE==6
! These routines don't exist in g6a/g65. Only publish if using 6.
      public g6_getnjmax
      public g6_set_j_particle_multisend_mxfast
#endif

#if GRAPE!=65
! These routines don't exist in g65. Only publish if using 6/6a.
      public g6_reinitialize
      public g6calc_firsthalf, g6calc_lasthalf
      public g6_set_j_particle_mxonly
#endif
#endif


!
! Number of boards, pipes per board, and how much j memory per board:
!

#if GRAPE==5 || GRAPE==565
      integer, parameter :: nboards=1
      integer, parameter :: boardnum=6
      integer, parameter :: nchips_per_board=8
      integer, parameter :: npipes_per_board=96
      integer, parameter :: gstorage=131072
#elif GRAPE==6
      integer, parameter :: nboards=1
      integer, parameter :: boardnum=6
      integer, parameter :: nchips_per_board=8
      integer, parameter :: npipes_per_board=48
      integer, parameter :: gstorage=262144
#elif GRAPE==65 || GRAPE==66
! grape6a
      integer, parameter :: nboards=1
      integer, parameter :: boardnum=6
      integer, parameter :: nchips_per_board=4
      integer, parameter :: npipes_per_board=48
      integer, parameter :: gstorage=131072      !or 65536
#endif


      integer, parameter :: maxpipes=npipes_per_board*nboards
      integer, parameter :: NJMAX=gstorage
      integer, parameter :: maxclusters=8
      integer, save:: in_use(0:maxclusters-1) = 0

!
! I wrote a small wait function to mock up how much delay
! there would be in communication. nspin sets how much
! delay there is.
!
!      integer, parameter:: nspin=50000
      integer, parameter:: nspin=0

      integer, save :: myboard
      integer, save :: niuse(0:maxclusters-1),njuse(0:maxclusters-1)
      double precision, save:: eps2

      integer, save :: g_jindx(0:NJMAX,0:maxclusters-1)
      double precision, save:: g_jstorage(4,0:NJMAX-1,0:maxclusters-1)


      integer, save :: g_iindx(maxpipes)
      double precision, save:: g_istorage(3,maxpipes)

      contains
!****************************************************************
!                                                               *
! Here are the routines                                         *
!                                                               *
!****************************************************************
!-----------------------------------------------------
      subroutine g5_openMC(n)
!
! Fake locking the grape board. 
! This should work for parallel operation.
!
      integer, intent(in):: n

      myboard=n

      if(in_use(n).eq.0)then
         in_use(n)=1
      else
         do while(in_use(n).ne.0)
           if(in_use(n).eq.0)then
              in_use(n)=1
              exit
           endif
         enddo
      endif

      end subroutine g5_openMC
!-----------------------------------------------------
      subroutine g5_open()
!
! Fake locking the grape board. 
! This should work for parallel operation.
!
      myboard=0

      if(in_use(0).eq.0)then
         in_use(0)=1
      else
         do while(in_use(0).ne.0)
           if(in_use(0).eq.0)then
              in_use(0)=1
              exit
           endif
         enddo
      endif

      end subroutine g5_open
!-----------------------------------------------------
      subroutine g5_closeMC(n)
!
! Fake unlocking the grape board. 
!
      integer, intent(in):: n

      in_use(n)=0

      end subroutine g5_closeMC
!-----------------------------------------------------
      subroutine g5_close()
!
! Fake unlocking the grape board. 
!
      in_use(0)=0

      end subroutine g5_close
!-----------------------------------------------------
      subroutine g5_set_eps_to_allMC(ic,eps)
      integer, intent(in):: ic
      double precision, intent(in):: eps

      eps2 = eps*eps

      end subroutine g5_set_eps_to_allMC
!-----------------------------------------------------
      subroutine g5_set_eps_to_all(eps)
      double precision, intent(in):: eps

      eps2 = eps*eps

      end subroutine g5_set_eps_to_all
!-----------------------------------------------------
      subroutine g5_set_rangeMC(ic,g5xmin,g5xmax,g5mmin)
      integer, intent(in):: ic
      double precision, intent(in):: g5xmin,g5xmax,g5mmin


      return
      end subroutine g5_set_rangeMC
!-----------------------------------------------------
      subroutine g5_set_range(g5xmin,g5xmax,g5mmin)
      double precision, intent(in):: g5xmin,g5xmax,g5mmin


      return
      end subroutine g5_set_range
!-----------------------------------------------------
      integer function g5_get_firstcluster()

      g5_get_firstcluster=boardnum

      return
      end function g5_get_firstcluster
!-----------------------------------------------------
      integer function g5_get_number_of_boards()


      g5_get_number_of_boards=nboards

      return
      end function g5_get_number_of_boards
!-----------------------------------------------------
      integer function g5_get_number_of_pipelines_per_board()

      g5_get_number_of_pipelines_per_board=npipes_per_board

      return
      end function
!-----------------------------------------------------
      subroutine g5_set_nMC(ic,nj)

      integer, intent(in):: ic,nj


      njuse(ic)=nj

      end subroutine g5_set_nMC
!-----------------------------------------------------
      subroutine g5_set_n(nj)

      integer, intent(in):: nj

      njuse(0)=nj

      end subroutine g5_set_n
!-----------------------------------------------------
      subroutine g5_set_xjMC(ic,j,nj,g5_xj)

      integer, intent(in):: ic,j,nj
      double precision, intent(in):: g5_xj(3,nj)

      integer jj,ncl
      double precision ddd

      call spin(3*nj*nspin,ddd)

      njuse(ic)=nj

      do ncl=ic,ic
         do jj=1,nj
            g_jstorage(1,j+jj-1,ncl)=g5_xj(1,jj)
            g_jstorage(2,j+jj-1,ncl)=g5_xj(2,jj)
            g_jstorage(3,j+jj-1,ncl)=g5_xj(3,jj)
         enddo
      enddo

      return
      end subroutine g5_set_xjMC
!-----------------------------------------------------
      subroutine g5_set_mjMC(ic,j,nj,g5_mj)

      integer, intent(in):: ic,j,nj
      double precision, intent(in):: g5_mj(nj)

      integer jj,ncl
      double precision ddd

      call spin(nj*nspin,ddd)

      njuse(ic)=nj

      do ncl=ic,ic
         do jj=1,nj
            g_jstorage(4,j+jj-1,ncl)=g5_mj(jj)
            g_jindx   (  j+jj-1,ncl)=-1
         enddo
      enddo

      return
      end subroutine g5_set_mjMC
!-----------------------------------------------------
      subroutine g5_set_xmj(j,nj,g5_xj,g5_mj)

      integer, intent(in):: j,nj
      double precision, intent(in):: g5_xj(3,nj)
      double precision, intent(in):: g5_mj(nj)

      integer jj,ncl
      double precision ddd

      call spin(3*nj*nspin,ddd)

      njuse(0)=nj

      do jj=1,nj
         g_jstorage(1,j+jj-1,0)=g5_xj(1,jj)
         g_jstorage(2,j+jj-1,0)=g5_xj(2,jj)
         g_jstorage(3,j+jj-1,0)=g5_xj(3,jj)
         g_jstorage(4,j+jj-1,0)=g5_mj(  jj)
         g_jindx   (  j+jj-1,0)=-1
      enddo

      return
      end subroutine g5_set_xmj
!-----------------------------------------------------
      subroutine g5_calculate_force_on_x(g5_xi,g5_acc,g5_phi,ni)

      integer, intent(in):: ni
      double precision, intent(in ):: g5_xi(3,ni)
      double precision, intent(out):: g5_acc(3,ni),g5_phi(ni)

      integer i,ierr
      double precision ddd
      double precision g6_v(3,maxpipes)
      double precision g6_j(3,maxpipes)
      double precision g6h (  maxpipes)


      niuse(myboard)=ni

      do i=1,ni
         call spin(nspin,ddd)      !time to send i data to grape

         g_istorage(1,i)=g5_xi(1,i)
         g_istorage(2,i)=g5_xi(2,i)
         g_istorage(3,i)=g5_xi(3,i)
         g_iindx     (i)=1

         call spin(nspin,ddd)      !time to get forces from grape

      enddo


      ierr= g6calc_lasthalf(myboard,njuse(myboard),niuse(myboard),
     &                      g_iindx,g_istorage,g6_v,
     &                      eps2,g6h,g5_acc,g6_j,g5_phi)


      return
      end subroutine g5_calculate_force_on_x
!-----------------------------------------------------
!*********************************************************************
!*********************************************************************
!*********************************************************************
!*****GRAPE 5 ROUTINES ABOVE, GRAPE 6 ROUTINES BELOW *****************
!*********************************************************************
!*********************************************************************
!*********************************************************************
!*********************************************************************
!-----------------------------------------------------
      integer function g6_npipes()

      g6_npipes=maxpipes

      return
      end function g6_npipes
!-----------------------------------------------------
      integer function g6_getnjmax()

      g6_getnjmax=NJMAX

      return
      end function g6_getnjmax
!-----------------------------------------------------
      subroutine g6_reinitialize(n)
      integer, intent(in):: n

      return
      end subroutine g6_reinitialize
!-----------------------------------------------------
      subroutine g6_set_xunit(n)
      integer, intent(in):: n

      return
      end subroutine g6_set_xunit
!-----------------------------------------------------
      subroutine g6_set_tunit(n)
      integer, intent(in):: n

      return
      end subroutine g6_set_tunit
!-----------------------------------------------------
      subroutine g6_set_ti(n,z)
      integer, intent(in):: n
      double precision, intent(in):: z

      return
      end subroutine g6_set_ti
!-----------------------------------------------------
      subroutine g6_set_nip(iboard,nip)
      integer, intent(in):: iboard, nip

      niuse(iboard)=nip

      return
      end subroutine g6_set_nip
!-----------------------------------------------------
      subroutine g6_set_njp(iboard,njp)
      integer, intent(in):: iboard, njp

      njuse(iboard)=njp

      return
      end subroutine g6_set_njp
!-----------------------------------------------------
      subroutine g6_open(n)
!
! Fake locking the grape board. 
! This should work for parallel operation.
!
      integer, intent(in):: n

      myboard=n

      if(in_use(n).eq.0)then
         in_use(n)=1
      else
         do while(in_use(n).ne.0)
           if(in_use(n).eq.0)then
              in_use(n)=1
              exit
           endif
         enddo
      endif

      end subroutine g6_open
!-----------------------------------------------------
      subroutine g6_close(n)
!
! Fake unlocking the grape board. 
!
      integer, intent(in):: n

      in_use(n)=0

      end subroutine g6_close
!-----------------------------------------------------
      subroutine g6_set_i_particle_scales_from_real_value(iboard,i,
     &                             g6_a,g6_j,g6_phi,a,b)
!****************************************************************
!                                                               *
! This routine is not required by fakegrape, since it doesn't   *
! need to set any scales                                        *
!                                                               *
!****************************************************************

      integer, intent(in):: iboard,i
      double precision, intent(in):: g6_a(3),g6_j(3),g6_phi
      double precision, intent(in):: a,b


      return
      end subroutine g6_set_i_particle_scales_from_real_value
!-----------------------------------------------------
      subroutine g6_set_i_particle(iboard,i,g6_i,
     &                             g6_x,g6_v,g6eps2,g6h)
!****************************************************************
!                                                               *
! This routine stores one particle in the i arrays, to use      *
! as a sink for calculating gravity                             *
!                                                               *
!****************************************************************

      integer, intent(in):: iboard,i,g6_i
      double precision, intent(in):: g6_x(3),g6_v(3),g6h,g6eps2

      eps2=g6eps2

      g_istorage(1,i+1)=g6_x(1)
      g_istorage(2,i+1)=g6_x(2)
      g_istorage(3,i+1)=g6_x(3)
      g_iindx     (i+1)=g6_i


      return
      end subroutine g6_set_i_particle
!-----------------------------------------------------
      subroutine g6_set_j_particle(iboard,j,i,g6_t,g6_dt,em,
     &               g6_ja2by18,g6_ja1by6,g6_jaby2,g6_jvel,xyz)
!****************************************************************
!                                                               *
! This routine stores one particle in the j arrays, to use      *
! as a source for calculating gravity                           *
!                                                               *
!****************************************************************

      integer, intent(in):: iboard,j,i
      double precision, intent(in):: em,xyz(3)

      double precision, intent(in):: g6_t,g6_dt,g6_ja2by18(3)
      double precision, intent(in):: g6_ja1by6(3),g6_jaby2(3)
      double precision, intent(in):: g6_jvel(3)

      double precision ddd

      if(j.gt.NJMAX)stop'Fakegrape6:Too many j particles'

      call spin(nspin,ddd)

      call  g6_set_j_particle_mxonly(iboard,j,i,em,xyz)

      return
      end subroutine g6_set_j_particle
!-----------------------------------------------------
      subroutine g6_set_j_particle_multisend_mxfast(iboard,nc,j,i,
     &                em,xyz)
!****************************************************************
!                                                               *
! This routine stores one particle in the j arrays, to use      *
! as a source for calculating gravity                           *
!                                                               *
!****************************************************************

      integer, intent(in):: iboard,nc,j,i
      double precision, intent(in):: em,xyz(3)

      integer ncl
      double precision ddd

      if(j.gt.NJMAX)stop'Fakegrape6:Too many j particles'

      call spin(nspin,ddd)

      do ncl=0,nc-1
         g_jstorage(1,j,ncl)=xyz(1)
         g_jstorage(2,j,ncl)=xyz(2)
         g_jstorage(3,j,ncl)=xyz(3)
         g_jstorage(4,j,ncl)=em
         g_jindx     (j,ncl)=i
      enddo

      return
      end subroutine g6_set_j_particle_multisend_mxfast
!-----------------------------------------------------
      subroutine g6_set_j_particle_mxonly(iboard,j,i,em,xyz)
!****************************************************************
!                                                               *
! This routine stores one particle in the j arrays, to use      *
! as a source for calculating gravity                           *
!                                                               *
!****************************************************************

      integer, intent(in):: iboard,j,i
      double precision, intent(in):: em,xyz(3)

      integer ncl
      double precision ddd

      if(j.gt.NJMAX)stop'Fakegrape6:Too many j particles'

      call spin(nspin,ddd)

      do ncl=iboard,iboard
         g_jstorage(1,j,ncl)=xyz(1)
         g_jstorage(2,j,ncl)=xyz(2)
         g_jstorage(3,j,ncl)=xyz(3)
         g_jstorage(4,j,ncl)=em
         g_jindx     (j,ncl)=i
      enddo

      return
      end subroutine g6_set_j_particle_mxonly
!-----------------------------------------------------
      subroutine g6calc_firsthalf(iboard,nj,ni,g6_i,g6_x,g6_v,
     &               g6_a,g6_j,g6_phi,g6eps2,g6h)
!****************************************************************
!                                                               *
! This routine pretends to be the first half of the gravity     *
! calculation. It stores the i particles on which gravity       *
! is to be calculated.                                          *
!                                                               *
!****************************************************************

      integer, intent(in):: iboard,nj,ni,g6_i(ni)
      double precision, intent(in):: g6_x(3,ni), g6_v(3,ni)
      double precision, intent(in):: g6_a(3,ni),g6_j(3,ni)
      double precision, intent(in):: g6_phi(ni),g6h(ni)
      double precision, intent(in):: g6eps2

      integer i
      double precision ddd

      call g6_set_nip(iboard,ni)
      call g6_set_njp(iboard,nj)

      do i=1,ni
         call spin(nspin,ddd)      !time to send i data to grape

         call g6_set_i_particle(iboard,i-1,g6_i(i),g6_x(:,i),
     &                          g6_v(:,i),g6eps2,g6h(i))

      enddo

      return
      end subroutine g6calc_firsthalf
!-----------------------------------------------------
      integer function g6calc_lasthalf(iboard,nj,ni,g6_i,g6_x,g6_v,
     &                  g6eps2,g6h,g6_a,g6_j,g6_phi)
!****************************************************************
!                                                               *
! This routine pretends to be the second half of the gravity    *
! calculation. It actually does the calculation.                *
!                                                               *
!****************************************************************

      integer, intent(in):: iboard,nj,ni,g6_i(ni)
      double precision, intent(in ):: g6eps2,g6h(ni)
      double precision, intent(in ):: g6_x(3,ni),g6_v(3,ni)
      double precision, intent(out):: g6_a(3,ni),g6_j(3,ni)
      double precision, intent(out):: g6_phi(ni)

      integer flag(maxpipes)
      double precision ddd


      eps2=g6eps2
      g6calc_lasthalf= g6_get_force(iboard,g6_a,g6_j,g6_phi,flag)

      return
      end function g6calc_lasthalf
!-----------------------------------------------------
      integer function g6_get_force(iboard,g6_a,g6_j,g6_phi,flag)
!****************************************************************
!                                                               *
! This routine pretends to be the second half of the gravity    *
! calculation. It actually does the calculation.                *
!                                                               *
!****************************************************************

      integer, intent(in ):: iboard
      integer, intent(out):: flag(maxpipes)
      double precision, intent(out):: g6_a(3,maxpipes),g6_j(3,maxpipes)
      double precision, intent(out):: g6_phi(maxpipes)

      integer i,j
      integer nlo1,nlo2,nhi1,nhi2,ii
      double precision dx,dy,dz,r2,rinv,r3inv,emr3inv
      double precision ddd

      g6_a=ZERO ; g6_j=ZERO ; g6_phi=ZERO ; flag=1

      do j=0,njuse(iboard)-1
         ii=0

         do i=1,niuse(iboard)
            if(g_iindx(i).eq.g_jindx(j,iboard))ii=i   !avoid self calculation
         enddo

         nlo1=1 ; nhi1=niuse(iboard)
         nlo2=1 ; nhi2=0

         if(ii.ne.0)then
            nlo1=1    ; nhi1=ii-1
            nlo2=ii+1 ; nhi2=niuse(iboard)
         endif

         do i=nlo1,nhi1

            dx=g_jstorage(1,j,iboard)-g_istorage(1,i)
            dy=g_jstorage(2,j,iboard)-g_istorage(2,i)
            dz=g_jstorage(3,j,iboard)-g_istorage(3,i)

            r2=dx**2 + dy**2 + dz**2 + eps2
            rinv=ONE/sqrt(r2)
            r3inv=rinv**3

            emr3inv=g_jstorage(4,j,iboard)*r3inv

            g6_a(1,i) = g6_a(1,i) + dx*emr3inv
            g6_a(2,i) = g6_a(2,i) + dy*emr3inv
            g6_a(3,i) = g6_a(3,i) + dz*emr3inv

            g6_phi(i) = g6_phi(i) - g_jstorage(4,j,iboard)*rinv
         enddo

         do i=nlo2,nhi2

            dx=g_jstorage(1,j,iboard)-g_istorage(1,i)
            dy=g_jstorage(2,j,iboard)-g_istorage(2,i)
            dz=g_jstorage(3,j,iboard)-g_istorage(3,i)

            r2=dx**2 + dy**2 + dz**2 + eps2
            rinv=ONE/sqrt(r2)
            r3inv=rinv**3

            emr3inv=g_jstorage(4,j,iboard)*r3inv

            g6_a(1,i) = g6_a(1,i) + dx*emr3inv
            g6_a(2,i) = g6_a(2,i) + dy*emr3inv
            g6_a(3,i) = g6_a(3,i) + dz*emr3inv

            g6_phi(i) = g6_phi(i) - g_jstorage(4,j,iboard)*rinv
         enddo

      enddo

      g6_get_force=0

      do i=1,niuse(iboard)
         call spin(nspin,ddd)      !time to get forces from grape
      enddo

      return
      end function g6_get_force
!-----------------------------------------------------
      end module fakegrape
