program sphere
!****************************************************************
! This module is a component of VINE, copyright by the authors, *
! as listed below. Please see the COPYING and LICENSE files     *
! in the parent directory for additional information.           *
! This notification is present as an attempt to follow the      *
! recommendations for licensing published on the FSF page.      * 
!*****************************************************************
! This is a template routine for creating initial condition      *
! dump files for VINE. It makes a homogeneous sphere of particles*
!                                                                *
! This set up is guaranteed not to do anything sensible.         *
! It is nothing more than a template, from which you can create  *
! routines of your own to do your own initial conditions.        *
!                                                                *
! Author: Andy Nelson 2008                                       *
!                                                                *
!*****************************************************************
use dimens
use partvars
use indivtsvars
use clmpvars
use miscvars
use sphvars
use constants

use treemake, only:mtree
use tree, only:activate_tree
use io
use smoothadjust
#ifdef SPH
   use sph, only: SPHkerntablinit
#endif
use stubs

implicit none

integer idump
integer i,np,np_sph,np_nbod,npm
integer konvert_to_physical_units
integer check_for_old_file
integer say_dumpfile_name

double precision emsys,rsys,empart,hpart
double precision ranpos(ndim),r2


!**************************************************************
! Initialize some scalars from the insph file
!**************************************************************

      call lunit
      call options(idump)

!**************************************************************
! Set the initial dimensions of your sphere
!**************************************************************

      print *,' '
      print *,' '
      print *,' '

      np_sph=-1 ; np_nbod=-1 ; npm=-1
      gamma=ONE

#ifdef PM
      do while(npm.ge.ipmdim .or. npm.lt.0)
         print *,'how many additional point masses do you want?'
         read *,npm
         if(npm.gt.ipmdim)then
            print *,'You set the code maximum to: ',ipmdim
         endif
      enddo
#endif

#ifdef SPH
      do while(np_sph.ge.isphdim .or. np_sph.lt.0)
         print *,'how many SPH particles do you want?'
         read *,np_sph
         if(np_sph.gt.isphdim)then
            print *,'You set the code maximum to: ',isphdim
         endif
      enddo
      print *,'what is the ratio of specific heats?'
      read  *,gamma
#endif

      do while(np_nbod.ge.nboddim .or. np_nbod.lt.0)
         print *,'how many Nbody particles do you want?'
         read *,np_nbod
         if(np_nbod.gt.nboddim)then
            print *,'You set the code maximum to: ',nboddim
         endif
      enddo

      npm    =max(npm,0)
      np_sph =max(np_sph,0)
      np_nbod=max(np_nbod,0)

      np= np_sph + np_nbod

      if(np.eq.0 .and. npm.eq.0)stop 'You have no particles'


      print *,'what is the total mass of your sphere?'
      read *,emsys

      print *,'what is the radius of your sphere?' 
      read *,rsys


      empart = emsys/dble(np)
      hpart  = (rsys**3/dble(np))**onethd


!*************************************************************
!
! Now start putting data into the appropriate arrays
!
!*************************************************************
!
! SPH particles
!
      npart=1
      do while(npart.le.np_sph)
         call random_number(ranpos)
         
         posmas(1,npart)=dble(ranpos(1)-HALF)*rsys*TWO
         posmas(2,npart)=dble(ranpos(2)-HALF)*rsys*TWO
         r2=posmas(1,npart)**2 + posmas(2,npart)**2
#if NDIM==3
         posmas(3,npart)=dble(ranpos(3)-HALF)*rsys*TWO
         r2 = r2 + posmas(3,npart)**2
#endif
         if(sqrt(r2).gt.rsys)cycle

         ipindx(npart)=npart
         itstepbin(npart)=0
         material  (npart)= 1

         alpha (    npart)=ONE
         u     (    npart)=ONE
         rho   (    npart)=ONE
         h     (    npart)=hpart
         poten (    npart)=ZERO
         posmas(ims,npart)=empart
         vel   (1  ,npart)=zero
         vel   (2  ,npart)=zero
#if NDIM==3
         vel   (3  ,npart)=zero
#endif
         npart=npart+1
      enddo
      npart_sph=np_sph
      npart    =np_sph

      if(npart_sph.gt.0)then

         call mtree(isoft,igrape,npart,npart_sph,sepmax,bsepmax,eps,  &
                    boxlims,posmas,h,numclumps,lstclumps)

         do i=1,npart
            iactive(i)=i
         enddo
         nactive=npart

         call activate_tree(igrape,numclumps,lstclumps,               &
                            nactive,iactive,nactive_c,iactive_c)

         call SPHkerntablinit

         call hadjust
      endif
      npart=npart_sph+1
!
! Nbody particles
!
      do while(npart.le.np)
         call random_number(ranpos)
         
         posmas(1,npart)=dble(ranpos(1)-HALF)*rsys*TWO
         posmas(2,npart)=dble(ranpos(2)-HALF)*rsys*TWO
         r2=posmas(1,npart)**2 + posmas(2,npart)**2
#if NDIM==3
         posmas(3,npart)=dble(ranpos(3)-HALF)*rsys*TWO
         r2 = r2 + posmas(3,npart)**2
#endif
         if(sqrt(r2).gt.rsys)cycle

         ipindx(npart)=npart
         itstepbin(npart)=0

         h     (    npart)=hpart
         posmas(ims,npart)=empart
         vel   (1  ,npart)=zero
         vel   (2  ,npart)=zero
#if NDIM==3
         vel   (3  ,npart)=zero
#endif
         npart=npart+1
      enddo
      npart=np
!
! Point masses
!
      npoim=1
      do while(npoim.le.npm)
         call random_number(ranpos)
         
         posmaspm(1,npart)=dble(ranpos(1)-HALF)*rsys*TWO
         posmaspm(2,npart)=dble(ranpos(2)-HALF)*rsys*TWO
         r2=posmaspm(1,npart)**2 + posmaspm(2,npart)**2
#if NDIM==3
         posmaspm(3,npart)=dble(ranpos(3)-HALF)*rsys*TWO
         r2 = r2 + posmaspm(3,npart)**2
#endif
         if(sqrt(r2).gt.rsys)cycle

         itstepbinpm(npart)=0

         hpm   (    npart)=hpart
         posmas(ims,npart)=HALF
         velpm (1  ,npart)=zero
         velpm (2  ,npart)=zero
#if NDIM==3
         velpm (3  ,npart)=zero
#endif
         npoim=npoim+1
      enddo
      npoim=npm

!**********************************************************
!
! And finally write data into file
!
!**********************************************************

      t=ZERO 
      deltnew=ONE

      nstep=0
      n_dtmax_steps=0
      n_ana_steps  =0
!
! If you set up your problem above with physical units already,
! don't convert them into physical units again, when you write 
! out your initial dump now. See also insph variables xmas,ylen
! and iunits
!
! Checking for an old file (and not finding one) means that we
! will write to file called `vinetmpdump'. Otherwise, we'll write
! to a file as defined by the `basenm' parameter that you set
! in your insph file. 
!
      konvert_to_physical_units=0
      check_for_old_file=0
      say_dumpfile_name=1

      call getfname(idump,check_for_old_file,say_dumpfile_name)

      call writefiles(idump,.false.,konvert_to_physical_units)

      stop
end
