      module constants
!****************************************************************
! This module is a component of VINE, copyright by the authors, *
! as listed below. Please see the COPYING and LICENSE files     *
! in the parent directory for additional information.           *
! This notification is present as an attempt to follow the      *
! recommendations for licensing published on the FSF page.      * 
!*********************************************************************
!  This file contains the VINE module defining lots of physical      *
!  constants.                                                        *
!                                                                    *
!  Written by Andy Nelson                                            *
!*********************************************************************
      implicit none
      public
      save
!
! numerical constants, small and big are set so that
! they can be squared without overflowing. 
!
      double precision, parameter::small=1d-150,big=1d+150

      double precision, parameter::
     &      PI=3.141592653589793238d0,twoPI=2d0*PI,
     &  dlog2 =0.30102999566398119d0,dlog2inv=1d0/dlog2,
     &  dlog10=2.302585092994059d0,dlog10inv=1d0/dlog10,
     &   zero= 0d0,  one    =1d0,  two= 2d0,  three= 3d0,
     &   four= 4d0,  five   =5d0,  six= 6d0,  seven= 7d0,
     &  eight= 8d0,  nine   =9d0,  ten=10d0,
     &      half=0.5d0, thrhalf=1.5d0,
     &   fivhalf=2.5d0, sevhalf=3.5d0,
     & onethd= .3333333333333333d0,twothd= .6666666666666666d0,
     & forthd=1.3333333333333333d0,fivthd=1.6666666666666666d0,
     &                             atethd=2.6666666666666666d0,
     & fourth=0.25d0,thrfourth=0.75d0,ninefourth=2.25d0,
     & fourninth=.444444444444444d0

      character(len=26), parameter:: 
     &                   Caplets='ABCDEFGHIJKLMNOPQRSTUVWXYZ',
     &                   Smllets='abcdefghijklmnopqrstuvwxyz'
      character(len=10), parameter:: Numbers='1234567890'
!
!
! physical constants, from NIST02
!
!
      double precision, parameter:: 
     &              echarge  =1.60217653d-19, !Coulombs
     &              Navag    =6.0221415 d+23, !Num/mol
     &              kBoltz   =1.3806505 d-16, !erg/degK
     &              Rgascon  =8.314472  d+07, !erg/mol/degK
     &              amu      =1.66053886d-24, !gm
     &              Mprot    =1.67262171d-24, !gm
     &              Melect   =9.1093826 d-28, !gm
     &              StefBoltz=5.670400  d-05, !erg/cm^2/s/degK^4
     &              hplank   =6.6260693 d-27, !erg s
     &              clight   =2.99792458d+10, !cm/s
     &              bigG     =6.6742    d-08  !dyne*cm^2/gm^2=cm^3/gm/s^2
!
!
! astronomical quantities from Allen 4th edition
!
!
      double precision, parameter:: Lsun= 3.845d33  !erg/s

      double precision, parameter:: 
     &              Msun  = 1.9891d+33,       !gm
     &              Mjup  = 1.8987d+30,       !gm
     &              Msat  = 5.6851d+29,       !gm
     &              Muran = 8.6849d+28,       !gm
     &              Mnept = 1.0244d+29,       !gm
     &              Mearth= 5.9737d+27,       !gm
     &              Mmoon = 7.3483d+25        !gm
!
! distances (Rearth is mean--see Allen section 11.1)
!
      double precision, parameter::
     &              OneAu   =1.4959787066d+13,     !cm
     &              parsec  =3.0856776   d+18,     !cm
     &              Rsun    =6.95508     d+10,     !cm
     &              Rearth  =6.371000    d+08,     !cm 
     &              Rjupeq  =7.1492      d+09,     !cm
     &              Rjuppol =6.6854      d+09,     !cm
     &              Rsateq  =6.0268      d+09,     !cm
     &              Rsatpol =5.4364      d+09,     !cm
     &              Ruraneq =2.5559      d+09,     !cm
     &              Ruranpol=2.4973      d+09,     !cm
     &              Rnepteq =2.4764      d+09,     !cm
     &              Rneptpol=2.4340      d+09,     !cm
     &              a_jup = 5.20336301d+00*OneAU,  !cm
     &              a_sat = 9.53707032d+00*OneAU,  !cm
     &              a_uran=19.19126393d+00*OneAU,  !cm
     &              a_nept=30.06896348d+00*OneAU   !cm
!
! times
!
      double precision, parameter::
     &           year  =3.15569259747d+07,     !s
     &           day   =86400.       d+00,     !s
     &           Tjup  = 11.85652502d+00*year, !s
     &           Tsat  = 29.42351935d+00*year, !s
     &           Turan = 83.74749682d+00*year, !s
     &           Tnept =163.7232045 d+00*year, !s
     &           TspinJ=0.41354d+00     *day , !s
     &           TspinS=0.44401d+00     *day , !s
     &           TspinU=0.71833d+00     *day , !s (counterrot)
     &           TspinN=0.67125d+00     *day , !s
     &           omegaJ=twoPI/TspinJ         , !rad/s
     &           omegaS=twoPI/TspinS         , !rad/s
     &           omegaU=twoPI/TspinU         , !rad/s
     &           omegaN=twoPI/TspinN           !rad/s

!
!   Momenta (g cm^2 /s) 
!   Moments of inertia from
!        Hubbard & Marley 1989 Icarus (used polar value, C)
!        Hubbard etal 1991 Science 253 648
!
      double precision, parameter::
     &           AmomSolSys= 3.148d+50,
     &           SpinSun   = 1.63 d+48,
     &           SpinJup   =.26401d0*Mjup *Rjupeq**2 *omegaJ,
     &           SpinSat   =.22037d0*Msat *Rsateq**2 *omegaS,
     &           SpinUran  =.22680d0*Muran*Ruraneq**2*omegaU,
     &           SpinNept  =.24   d0*Mnept*Rnepteq**2*omegaN
!
! units
!
      double precision, parameter::
     &           janssky   = 1d-23,         !erg/cm^2/Hz/s (Fnu)
     &           asec      = 206264.8062d0, !asec per radian.
     &           deg_in_rad=57.29577951d0 


      end module constants
