      module dimens
!****************************************************************
! This module is a component of VINE, copyright by the authors, *
! as listed below. Please see the COPYING and LICENSE files     *
! in the parent directory for additional information.           *
! This notification is present as an attempt to follow the      *
! recommendations for licensing published on the FSF page.      * 
!****************************************************************
!  This file contains the VINE module to define a number of     *
!  basic parameters for array declarations, like the maximum    *
!  number of particles and dimensionality of the problem.       *
!                                                               *
!  Written by Andy Nelson, Markus Wetzstein and Thorsten Naab   *
!****************************************************************
      implicit none

      public
      save
!
! Precision information. Eventually convert the whole code
! to use these instead of defaults. qp comes from public
! domain code posted by James Van Buskirk on comp.lang.fortran
!
      integer, parameter:: sp = selected_real_kind( 6,  30)
      integer, parameter:: dp = selected_real_kind(15, 300)
      integer, parameter:: qp_pref=selected_real_kind(30,1000)
      integer, parameter:: qp = qp_pref*(1+sign(1,qp_pref))/2 +
     &                               dp*(1-sign(1,qp_pref))/2

      integer, parameter:: ik2 = selected_int_kind( 4)
      integer, parameter:: ik4 = selected_int_kind(10)
      integer, parameter:: ik8 = selected_int_kind(18)
!
!=================PARTICLE INFORMATION===================
!
! maximum number of SPH, Nbody and total particles
!  both isphdim and nboddim must be at least 1
!
#ifdef SPH
      integer, parameter:: isphdim=130000
#else
      integer, parameter:: isphdim=1
#endif
      integer, parameter:: nboddim=200000 
!      integer, parameter:: nboddim=1
      integer, parameter:: idim=isphdim+nboddim
      integer, parameter:: ipmdim=2
!
! number of dimensions
!
#if NDIM==2
      integer, parameter:: ndim=2
      double precision, parameter:: diminv=0.5d0
#else
      integer, parameter:: ndim=3
      double precision, parameter:: diminv=0.333333333333333333d0
#endif
      integer, parameter:: ndim1=ndim+1
      integer, parameter:: ndim2=ndim+2
      integer, parameter:: ndim3=ndim+3
!
! array locations for neighbor quantities
!
      integer, parameter:: idx=1,idy=2,idz=3,ims=ndim1
      integer, parameter:: ir2=ims+1,ih2=ims+2
!
! minimum and maximum number of neighbors allowed
! before limiting behavior sets in.
!
#if NDIM==2
! 2D
      integer, parameter:: neimin=10, neimax=30
#else
! 3D
      integer, parameter:: neimin=30, neimax=70
#endif
!
!================EOS INFORMATION====================
!
      integer, parameter :: id_idealgas  =0
      integer, parameter :: id_lagrgiso  =1
      integer, parameter :: id_euleriso  =2
      integer, parameter :: id_polyeos   =3
      integer, parameter :: id_entreos   =4
      integer, parameter :: id_bateeos   =5
!
!================BOUNDARY INFORMATION===============
!
      integer, parameter :: id_freebnd=0
      integer, parameter :: id_perdbnd=1
      integer, parameter :: id_reflbnd=2
!
!================TREE INFORMATION===================
!
! Identify softening type for gravity
!
      integer, parameter :: id_varkern=0
      integer, parameter :: id_fixkern=1
      integer, parameter :: id_Plummer=2
! 
! Identify MAC type
!
      integer, parameter :: id_angle=1
      integer, parameter :: id_SWabs=2
      integer, parameter :: id_gadget=3
!
! clump and bunch sizes
!
! Clumps are groups of nearby particles that are grouped together
! for a single traversal in the gravity and neighbor search SPH 
! calculations.
!
! Bunches are used only with the grape+tree option. They are 
! replacements for clumps during the tree walk for gravity, 
! since grapes like to get lots of particles at once, but
! doing treewalks for large clumps is not efficient for 
! other parts of the calculation. Clumps are still used for the
! non gravity parts (i.e. in sph), or if you turn off the 
! grape manually (igrape=0).
!
      integer, parameter :: maxclumpsize=neimax

      integer, parameter :: maxbunchsize=1000
!
! Maximum work to bite off at once, so as not to overrun
! arrays when doing neighbor searches etc.
!
      integer, parameter :: maxbite=2*max(neimax,maxclumpsize)
      integer, parameter :: maxsafe=max(0,maxbite-maxclumpsize)
!
!================MISC=============================
!
      integer, parameter:: maxcpus=512  !just big number.

      integer idbg,itme

      end module dimens
