      module grape
!****************************************************************
! This module is a component of VINE, copyright by the authors, *
! as listed below. Please see the COPYING and LICENSE files     *
! in the parent directory for additional information.           *
! This notification is present as an attempt to follow the      *
! recommendations for licensing published on the FSF page.      * 
!*********************************************************************
! This file contains the VINE module to do gravity using the         *
! GRAPE 6 special purpose hardware boards. The routines here         *
! are called from the forcelaw.F module.                             *
!                                                                    *
!  Written by Markus Wetzstein, Andy Nelson and Thorsten Naab        *
!*********************************************************************
#if GRAPE==6
#ifndef SELFFORCE
      grape6 module: Grape6 flag set, but self force flag is not'
#endif
#elif GRAPE==65
#ifndef SELFFORCE
      grape6 module: Grape65 flag set, but self force flag is not'
#endif
#elif GRAPE==66
#ifndef SELFFORCE
      grape6 module: Grape6a flag set, but self force flag is not'
#endif
#else
      grape6 module:No or wrong grape preprocessor flag exists!
#endif
      use dimens
      use treevars
      use indivtsvars
      use constants
      use perform
#if FAKE
      use fakegrape
#endif

      implicit none
      save

      private
      public unlock_grape,lock_grape,grape_tree,grape_dirsum
      public g6_oldacc,g6_oldjrk,g6_oldphi  !need public for reordering in orderparticles
!
! Functions from the grape library.
!
#if !FAKE
#if GRAPE==6
      integer  g6_npipes,g6calc_lasthalf,g6_getnjmax
      external g6_npipes,g6calc_lasthalf,g6_getnjmax
#elif GRAPE==65
      integer  g6_npipes,g6_get_force
      external g6_npipes,g6_get_force
#elif GRAPE==66
      integer  g6_npipes,g6calc_lasthalf
      external g6_npipes,g6calc_lasthalf
#endif
#endif

!
! Number of boards, pipes per board, and total possible pipes.
! yields how much j memory per board:
!
#if GRAPE==6
      integer, parameter :: nchips_per_board=8 
      integer, parameter :: gstorage=262144
#elif GRAPE==65 || GRAPE==66
      integer, parameter :: nchips_per_board=4 
      integer, parameter :: gstorage=131072      !or 65536
#endif

      integer, parameter :: iboard=6
      integer, parameter :: maxboards=1
      integer, parameter :: npipes_per_board=48
      integer, parameter :: maxpipes=npipes_per_board*maxboards
      integer, parameter :: NJMAX=gstorage
!
! Data for j particles to send:
!
      double precision g6_mj, g6_xj(3)
!
! Data for i particles to send:
!
      integer g6_iindx(maxpipes)
      double precision g6h      (  maxpipes),
     &                 g6_xi    (3,maxpipes),
     &                 g6_vi    (3,maxpipes), g6eps2
!
! Data for i particles to send, then receive:
!
      double precision g6_acc(3,maxpipes)
      double precision g6_jrk(3,maxpipes)
      double precision g6_phi(  maxpipes)
!
! Storage for old values
!
      double precision g6_oldacc(3,idim),
     &                 g6_oldjrk(3,idim),
     &                 g6_oldphi(  idim)
!
! Temporary array:
!
      double precision jrk(3,idim)

      double precision, parameter:: gconst=ONE

      contains
!-----------------------------------------------------------------------
!***********************************************************************
!***********************************************************************
!***********************************************************************
!***********************************************************************
#if GRAPE==65
!***********************************************************************
! Create routines for the ones that only exist in grape6/6a, but not   *
! in grape65                                                           *
!***********************************************************************
      integer function g6_getnjmax()

      g6_getnjmax=NJMAX

      return
      end function g6_getnjmax
!-----------------------------------------------------------------------
      subroutine g6_reinitialize(iboard)

      integer, intent(in):: iboard

      !this subroutine is empty in g6a/g65

      return
      end subroutine g6_reinitialize
!-----------------------------------------------------------------------
      subroutine g6_set_j_particle_mxonly(iboard,j,ii,g6_mj,g6_xj)

      integer, intent(in):: iboard,j,ii
      double precision, intent(in)::g6_mj,g6_xj(3)

      double precision, parameter:: g6_t=ZERO,g6_dt=ZERO
      double precision, parameter:: g6_ja2by18(3)=ZERO
      double precision, parameter:: g6_ja1by6 (3)=ZERO
      double precision, parameter:: g6_jaby2  (3)=ZERO
      double precision, parameter:: g6_jvel   (3)=ZERO


      call g6_set_j_particle(iboard,j,ii,g6_t,g6_dt,
     &                       g6_mj,g6_ja2by18,g6_ja1by6,g6_jaby2,
     &                       g6_jvel,g6_xj)

      end subroutine g6_set_j_particle_mxonly
!-----------------------------------------------------------------------
      subroutine g6calc_firsthalf(iboard,nj,ni,g6_iindx,
     &                   g6_xi,g6_vi,g6_acc,g6_jrk,g6_phi,
     &                   g6eps2,g6h)

      integer, intent(in):: iboard,nj,ni,g6_iindx(maxpipes)
      double precision, intent(in):: g6eps2
      double precision, intent(in):: g6_xi (3,maxpipes)
      double precision, intent(in):: g6_vi (3,maxpipes)
      double precision, intent(in):: g6_acc(3,maxpipes)
      double precision, intent(in):: g6_jrk(3,maxpipes)
      double precision, intent(in):: g6_phi(  maxpipes)
      double precision, intent(in):: g6h   (  maxpipes)

      integer ii

      stop'G65 option does not work'

      do ii=1,ni

         call g6_set_i_particle_scales_from_real_value(iboard,ii-1,
     &                  g6_acc(1,ii),g6_jrk(1,ii),g6_phi(ii),SIX,ZERO)

         call g6_set_i_particle(iboard,ii-1,g6_iindx(ii),g6_xi(1,ii),
     &                               g6_vi(1,ii),g6eps2,g6h(ii))

      enddo

      call g6_set_nip(iboard,ni)
      call g6_set_njp(iboard,nj)

      return
      end subroutine g6calc_firsthalf
!-----------------------------------------------------------------------
      integer function g6calc_lasthalf(iboard,nj,ni,g6_iindx,
     &                   g6_xi,g6_vi,
     &                   g6eps2,g6h,g6_acc,g6_jrk,g6_phi)

      integer, intent(in):: iboard,nj,ni,g6_iindx(maxpipes)
      double precision, intent(in ):: g6eps2
      double precision, intent(in ):: g6h   (  maxpipes)
      double precision, intent(in ):: g6_xi (3,maxpipes)
      double precision, intent(in ):: g6_vi (3,maxpipes)
      double precision, intent(out):: g6_acc(3,maxpipes)
      double precision, intent(out):: g6_jrk(3,maxpipes)
      double precision, intent(out):: g6_phi(  maxpipes)

      integer flag(maxpipes)

      g6calc_lasthalf = g6_get_force(iboard,g6_acc,g6_jrk,g6_phi,flag)

      return
      end function g6calc_lasthalf
!-----------------------------------------------------------------------
#endif
#if GRAPE==66
!***********************************************************************
! Create routines for the ones that only exist in grape6, but not      *
! in grape6a                                                           *
!***********************************************************************
      integer function g6_getnjmax()

      g6_getnjmax=NJMAX

      return
      end function g6_getnjmax
!-----------------------------------------------------------------------
#endif
!***********************************************************************
!***********************************************************************
!***Now start main portion of the module, where regular routines live***
!***********************************************************************
!***********************************************************************
!-----------------------------------------------------------------------
      subroutine lock_grape()

#ifdef _OPENMP
      print *,'You attempted to use both OpenMP and GRAPE!'
      print *,'Each GRAPE board must only be accessed by one thread.'
      print *,'Thus using OpenMP and GRAPE together would result in a'
      print *,'crash, as in this version of VINE the boards are'
      print *,'shared between the threads.'
      stop
#endif

      call g6_open(iboard)

      return
      end subroutine lock_grape
!-----------------------------------------------------------------------
      subroutine unlock_grape()

      call g6_close(iboard)

      return
      end subroutine unlock_grape
!-----------------------------------------------------------------------
      subroutine grape_tree(ibunch,isoft,eps2,npart,nact,listact,
     &                      nla0,nla,latoms,vel,apcorr,acc,poten)
!*************************************************************************
! This computes grav. accelerations on GRAPE-6. Board has to be opened   *
! outside this routine.                                                  *
!                                                                        *
! Written by M. Wetzstein, T. Naab, Apr 2002                             *
! Modifications by M. Wetzstein, Apr 2002, Dec. 2002                     *
! Modifications by A. Nelson,    Apr 2005 (code cleanup, here and dirsum)*
!                                                                        *
!*************************************************************************

! ibunch,npart aren't used here, they are just passed because 
! the call sequence has to be the same for all grape_* routines
! in the different grape module files

      integer, intent(in):: ibunch,isoft,npart
      integer, intent(in):: nact,listact(nact)
      integer, intent(in):: nla0,nla ,latoms(nla)
      double precision, intent(in   ):: eps2
      double precision, intent(in   ):: vel(3,npart)
      double precision, intent(inout):: apcorr(0:ndim,nact)
      double precision, intent(inout):: acc(ndim,npart)
      double precision, intent(inout):: poten(npart)

      integer i,ii,j,k
      integer ierr,nharderr,maxjstorage
      integer njlow,njhigh,nj
      integer nilow,nihigh,ni,npipes

      logical, save:: startup= .true.

      double precision tme1,tme2,tmeio1,tmeio2
      double precision tmecalc1,tmecalc2
      character(len=20), parameter:: where =('grape_tree')
      if(itme.ge.1)call cpu_time(tme1)


      if(isoft.ne.id_Plummer)then
         print *,'You have not selected Plummer softening'
         print *,'and you have selected grape. This is not'
         print *,'possible.'
         stop 'Not using Plummer softening with Grape'
      endif

      if(startup)then
         startup=.false.
!
! Initialize old values with educated guesses
!
         do i=1,npart
            g6_oldacc(1,i)=one
            g6_oldacc(2,i)=one
            g6_oldacc(3,i)=one
            g6_oldjrk(1,i)=1d2
            g6_oldjrk(2,i)=1d2
            g6_oldjrk(3,i)=1d2
            g6_oldphi(  i)=1d2
         enddo
      endif

      do j=1,nact 
         i=linktreetosph(listact(j))
         jrk(1,i)=zero
         jrk(2,i)=zero
         jrk(3,i)=zero
      enddo

      maxjstorage=g6_getnjmax()
      npipes=min(maxpipes,g6_npipes())

      nharderr=0 ; njlow=1
      noharderrors: do while(nharderr.le.20 .and. njlow.lt.nla)

         call setresolution(eps2)

         njlow=1 ; njhigh=min(nla,min(maxjstorage,NJMAX))
         do while(njlow.le.nla)

            if(itme.ge.1)call cpu_time(tmeio1)
!
! Cycle through groups of nj particles/nodes that contribute forces
!
            nj=njhigh-njlow+1
         
            do i=njlow,njhigh
               ii=latoms(i)
               j=i - njlow + 1

               g6_xj(1)=treept(1  ,ii)
               g6_xj(2)=treept(2  ,ii)
               g6_xj(3)=treept(3  ,ii)
               g6_mj   =treept(ims,ii)

               call g6_set_j_particle_mxonly(iboard,j-1,ii,g6_mj,g6_xj)
            enddo
            if(itme.ge.1)then
               call cpu_time(tmeio2)
               timing(itme_grpIO)=timing(itme_grpIO)+(tmeio2-tmeio1)
               call cpu_time(tmecalc1)
            endif

            nilow =1 ; nihigh=min(nact,npipes)
            do while(nilow.le.nact)
!
! Cycle through groups of ni particles that need forces
!
               ni= nihigh - nilow + 1
               do i=nilow,nihigh
                  ii = i - nilow +1
                  j=listact(i)
                  k=linktreetosph(j)

                  g6_iindx(  ii)=j
                  g6_xi   (1,ii)=treept   (1,j)
                  g6_xi   (2,ii)=treept   (2,j)
                  g6_xi   (3,ii)=treept   (3,j)
                  g6_vi   (1,ii)=vel      (1,k)
                  g6_vi   (2,ii)=vel      (2,k)
                  g6_vi   (3,ii)=vel      (3,k)
                  g6_acc  (1,ii)=g6_oldacc(1,k)
                  g6_acc  (2,ii)=g6_oldacc(2,k)
                  g6_acc  (3,ii)=g6_oldacc(3,k)
                  g6_jrk  (1,ii)=g6_oldjrk(1,k)
                  g6_jrk  (2,ii)=g6_oldjrk(2,k)
                  g6_jrk  (3,ii)=g6_oldjrk(3,k)
                  g6_phi  (  ii)=g6_oldphi(  k)
               enddo

               call g6calc_firsthalf(iboard,nj,ni,g6_iindx,
     &                   g6_xi,g6_vi,g6_acc,g6_jrk,g6_phi,
     &                   g6eps2,g6h)
               ierr=g6calc_lasthalf (iboard,nj,ni,g6_iindx,
     &                   g6_xi,g6_vi,
     &                   g6eps2,g6h,g6_acc,g6_jrk,g6_phi)

               if(ierr.eq.1)then
                  call g6_reinitialize(iboard)
                  nharderr=nharderr+1
                  if(nharderr.le.20)cycle noharderrors 

                  call g6_close(iboard)
                  stop'Too may hard errors on G6!!!'
               endif
         
               do i=nilow,nihigh
                  k=linktreetosph(listact(i))
                  ii = i - nilow + 1

                  acc  (1,k) = acc  (1,k) + gconst*g6_acc(1,ii)
                  acc  (2,k) = acc  (2,k) + gconst*g6_acc(2,ii)
                  acc  (3,k) = acc  (3,k) + gconst*g6_acc(3,ii)
                  jrk  (1,k) = jrk  (1,k) +        g6_jrk(1,ii)
                  jrk  (2,k) = jrk  (2,k) +        g6_jrk(2,ii)
                  jrk  (3,k) = jrk  (3,k) +        g6_jrk(3,ii)               
                  poten(  k) = poten(  k) - gconst*g6_phi(  ii)
               enddo

               nilow  = nihigh + 1
               nihigh = min(nact, nihigh + maxpipes)
            enddo

            if(itme.ge.1)then
               call cpu_time(tmecalc2)
               timing(itme_grpcalc)=timing(itme_grpcalc)+
     &                                (tmecalc2-tmecalc1)
            endif

            njlow  = njhigh + 1
            njhigh = min(nla, njhigh + min(maxjstorage,NJMAX))
         enddo
      enddo noharderrors

      if(nharderr.ne.0)print *,'G6 hard errors:',nharderr

!     store old quantities 

      do i=1,nact
         j=linktreetosph(listact(i))
         g6_oldacc(1,j) = acc(1,j)
         g6_oldacc(2,j) = acc(2,j)
         g6_oldacc(3,j) = acc(3,j)
         g6_oldjrk(1,j) = jrk(1,j)
         g6_oldjrk(2,j) = jrk(2,j)
         g6_oldjrk(3,j) = jrk(3,j)
         g6_oldphi(  j) = poten(j)
      enddo

      if(itme.ge.1)then
         call cpu_time(tme2)
         timing(itme_grape)=timing(itme_grape)+(tme2-tme1)
         ncalls(itme_grape)=ncalls(itme_grape)+1
      endif

      return
      end subroutine grape_tree
!------------------------------------------------------------------------
      subroutine grape_dirsum(npart,isoft,eps2,posmas,vel,acc,poten)

      integer, intent(in):: npart,isoft
      double precision, intent(in   ):: eps2
      double precision, intent(in   ):: posmas(ndim1,npart)
      double precision, intent(in   ):: vel(ndim,npart)
      double precision, intent(inout):: acc(ndim,npart)
      double precision, intent(inout):: poten(npart)

      integer i,ii,j,jj
      integer njlow,njhigh,nj
      integer nilow,nihigh,ni,npipes

      integer ni1,ni2,ni_inc

      integer ierr,nharderr,maxjstorage

      logical, save:: startup= .true.

      double precision tme1,tme2,tmeio1,tmeio2
      double precision tmecalc1,tmecalc2
      character(len=20), parameter:: where =('grape_dirsum')
      if(idbg.ge.2)print *,where
      if(itme.ge.1)call cpu_time(tme1)


      if(isoft.ne.id_Plummer)then
         print *,'You have not selected Plummer softening'
         print *,'and you have selected grape. This is not'
         print *,'possible.'
         stop 'Not using Plummer softening with Grape'
      endif

      if(startup)then
         startup=.false.
!
! Initialize old values with educated guesses
!
         do i=1,npart
            g6_oldacc(1,i)=one
            g6_oldacc(2,i)=one
            g6_oldacc(3,i)=one
            g6_oldjrk(1,i)=1d2
            g6_oldjrk(2,i)=1d2
            g6_oldjrk(3,i)=1d2
            g6_oldphi(  i)=1d2
         enddo
      endif

      maxjstorage=g6_getnjmax()
      npipes=min(maxpipes,g6_npipes())

      nharderr=0 ; njlow=1
      noharderrors: do while(nharderr.le.20 .and. njlow.lt.npart)

         do j=1,nactive
            i=iactive(j)
            jrk(1,i)=zero
            jrk(2,i)=zero
            jrk(3,i)=zero
         enddo

         call setresolution(eps2)

         njlow =1 ; njhigh=min(maxjstorage,NJMAX,npart)

         do while(njlow.le.npart)
            if(itme.ge.1)call cpu_time(tmeio1)
!
! Cycle through groups of nj particles that contribute forces
!
            nj=njhigh-njlow+1

            do jj=njlow,njhigh
               j=jj-njlow+1

               g6_xj(1)=posmas(1  ,jj)
               g6_xj(2)=posmas(2  ,jj)
               g6_xj(3)=posmas(3  ,jj)
               g6_mj   =posmas(ims,jj)

               call g6_set_j_particle_mxonly(iboard,j-1,jj,g6_mj,g6_xj)
            enddo

            if(itme.ge.1)then
               call cpu_time(tmeio2)
               timing(itme_grpIO)=timing(itme_grpIO)+(tmeio2-tmeio1)
               call cpu_time(tmecalc1)
            endif

            nilow =1 ; nihigh=min(nactive,npipes)
            do while(nilow.le.nactive)
!
! Cycle through groups of ni particles that need forces
!
               ni=nihigh - nilow + 1
               do i=nilow,nihigh
                  ii=i - nilow + 1
                  j=iactive(i)
                  g6_iindx(  ii)=            j
                  g6_xi   (1,ii)=posmas   (1,j)
                  g6_xi   (2,ii)=posmas   (2,j)
                  g6_xi   (3,ii)=posmas   (3,j)
                  g6_vi   (1,ii)=vel      (1,j)
                  g6_vi   (2,ii)=vel      (2,j)
                  g6_vi   (3,ii)=vel      (3,j)
                  g6_acc  (1,ii)=g6_oldacc(1,j)
                  g6_acc  (2,ii)=g6_oldacc(2,j)
                  g6_acc  (3,ii)=g6_oldacc(3,j)
                  g6_jrk  (1,ii)=g6_oldjrk(1,j)
                  g6_jrk  (2,ii)=g6_oldjrk(2,j)
                  g6_jrk  (3,ii)=g6_oldjrk(3,j)
                  g6_phi  (  ii)=g6_oldphi(  j)
               enddo

               call g6calc_firsthalf(iboard,nj,ni,g6_iindx,
     &                  g6_xi,g6_vi,g6_acc,g6_jrk,g6_phi,
     &                  g6eps2,g6h)
               ierr=g6calc_lasthalf (iboard,nj,ni,g6_iindx,
     &                  g6_xi,g6_vi,
     &                  g6eps2,g6h,g6_acc,g6_jrk,g6_phi)

               if(ierr.eq.1)then
                  call g6_reinitialize(iboard)
                  nharderr=nharderr+1
                  if(nharderr.le.20)cycle noharderrors 

                  call g6_close(iboard)
                  stop'Too may hard errors on G6!!!'
               endif

               do i=nilow,nihigh
                  ii=i - nilow + 1
                  j=iactive(i)
                  acc  (1,j) = acc  (1,j) + gconst*g6_acc(1,ii)
                  acc  (2,j) = acc  (2,j) + gconst*g6_acc(2,ii)
                  acc  (3,j) = acc  (3,j) + gconst*g6_acc(3,ii)
                  jrk  (1,j) = jrk  (1,j) +        g6_jrk(1,ii)
                  jrk  (2,j) = jrk  (2,j) +        g6_jrk(2,ii)
                  jrk  (3,j) = jrk  (3,j) +        g6_jrk(3,ii)
                  poten(  j) = poten(  j) - gconst*g6_phi(  ii)
               enddo

               nilow  = nihigh + 1
               nihigh = min(nactive, nihigh + npipes)
            enddo
            if(itme.ge.1)then
               call cpu_time(tmecalc2)
               timing(itme_grpcalc)=timing(itme_grpcalc)+
     &                                (tmecalc2-tmecalc1)
            endif


            njlow  = njhigh + 1
            njhigh = min(npart, njhigh + min(NJMAX,maxjstorage))
         enddo

      enddo noharderrors
      if(nharderr.ne.0)print *,'G6 hard errors:',nharderr

! store old quantities 

      do i=1,nactive
         j=iactive(i)
         g6_oldacc(1,j) = acc  (1,j)
         g6_oldacc(2,j) = acc  (2,j)
         g6_oldacc(3,j) = acc  (3,j)
         g6_oldjrk(1,j) = jrk  (1,j)
         g6_oldjrk(2,j) = jrk  (2,j)
         g6_oldjrk(3,j) = jrk  (3,j)
         g6_oldphi(  j) = poten(  j)
      enddo         
      
      if(itme.ge.1)then
         call cpu_time(tme2)
         timing(itme_grape)=timing(itme_grape)+(tme2-tme1)
         ncalls(itme_grape)=ncalls(itme_grape)+1
         if(itme.ge.2)print *,where,xcpus_inv*(tme2-tme1)
      endif

      return
      end subroutine grape_dirsum
!------------------------------------------------------------------------
      subroutine setresolution(eps2)
!
! Set the default precsion as recommended by the grape6
! manual:
!         51 bits to represent fraction part in positions
!         51 bits to represent fraction part in time
!         time 0.d0, no prediction
!
! Also set softening and smoothing values
!
      double precision, intent(in):: eps2

      integer i

      call g6_set_xunit(51)
      call g6_set_tunit(51)
      call g6_set_ti(iboard,zero)
!
! prepare softening and smoothing lengths
!
      g6eps2=eps2
      do i=1,maxpipes
         g6h(i)=zero
      enddo

      return
      end subroutine setresolution
!------------------------------------------------------------------------
      end module grape
