      module miscvars
!****************************************************************
! This module is a component of VINE, copyright by the authors, *
! as listed below. Please see the COPYING and LICENSE files     *
! in the parent directory for additional information.           *
! This notification is present as an attempt to follow the      *
! recommendations for licensing published on the FSF page.      * 
!*********************************************************************
!  This file contains the VINE module to define various switches,    *
!  settings and parameters that are important for the code to run    *
!                                                                    *
!  Written by Andy Nelson, Markus Wetzstein and Thorsten Naab        *
!*********************************************************************
      use dimens, only:ndim

      implicit none

      save
      private ndim
      public
!
! Switches
!
      integer
     &   io_fmt,iunits,
     &   ipres,ipdv,                                  !sph
     &   ieos,ivisc,ivtime,ibals,ishok,icool,iheat,   !thermo
     &   igrav,imac,isoft,igrape,igrapelim,idirect,   !grav
     &   irevise,
     &   indts,iusebin,i_hts,ifullextrap,             !stepping     
     &   ibctype,ishift,                              !boundary
     &   iexpand,                                     !cosmo
     &   irem,imovpm,                                 !PM/particles
     &   idustev,istrheat,icloudheat,                 !Disk stuff
     &   makechkpnt                                   !Make checkpoint dumps

!
! Settings
!
      double precision
     &     hmin,hmax,umin,taucool,                             !sph
     &     alphamax,alphastar,betastar,vlength,                !thermo
     &     treeacc,eps,sepmax,bsepmax,gmasslim,                !grav
     &     cfl, dhmax, tol, ftol, vtol, tolpm, ftolpm, vtolpm, !stepping err
     &     xmin,xmax,ymin,ymax,zmin,zmax,                      !boundary values
     &     boxlims(2,ndim),boxlen(ndim),halfboxlen(ndim),      !boundary box
     &     rdisk,colfrac
!
! tree rebuild settings
! Note that nbuild is defined differently for global or indivts.
! Global: timesteps since rebuild.
! Indts : number of builds since last reordering
!
      logical rebuild,hclmp_is_exact
      integer nbuild,maxbuild
      double precision clhfrac

!
! Multiple particle detection and statistics
!
      integer, parameter:: maxmult=10
      integer nmulttot(maxmult)
      integer, parameter:: kern_incr=1000
      integer neigdistrib(kern_incr)
      double precision vstep(kern_incr)
      double precision, parameter:: xkern_incr=kern_incr
      double precision, parameter:: vkernmax=2d0     !hardwired to W4kern
      double precision, parameter:: dvkern=vkernmax/xkern_incr
!
! Energy and EOS 
!
      double precision etot, ekin, egrav, etherm
      double precision uconst, uexpon, rcore
      double precision, parameter:: Avemwt=2.31d0 !un-ionized solar
      double precision gamma
      double precision rhozero,eosK
!
! Input, output and job control.
!
      integer iterm, idisk1, idisk2, idisk3, idisk4

      integer, parameter :: len_basenm=45
      integer, parameter :: len_abspath=200
      integer, parameter :: len_dumpdigits=4
      integer, parameter :: len_dumpfilename=
     &                        len_abspath+len_basenm+len_dumpdigits 
      character(len=len_abspath     ) absolutepath
      character(len=len_basenm      ) basenm
      character(len=len_dumpfilename) dumpfile
      character(len=14), parameter:: chkpntdumpname=('VINEcheckpoint')
      character(len=11), parameter:: nrfile =('lastdumpnum')


      integer, parameter:: idumpchkpnt=10000   ! File descriptor for 
                                               ! opening a checkpoint dump

!
! Current time, output or analysis dump time, etc. 
!
      logical readfromchkpnt   !dump we read in was a checkpoint dump 
      logical override         !override checkpoint dump's idea of 
                               !next output time 

      integer nstepmax          !simulation maxima
      double precision tstop

      integer nstep 
      integer idump0_out, n_dtmax_steps, next_outstep, nout_max
      integer idump0_ana, n_ana_steps  , next_anastep, nana_max

      double precision t0_out, tnext_out, tlast_out, dt_out
      double precision t0_ana, tnext_ana, tlast_ana, dt_ana
      double precision dtinit, t, tnow, t_deriv, t_endts,
     &          dtmin, dtmax, deltold, deltnew, dtnext
!
! conversion factors
!
      double precision xmas, ylen

      double precision d_unit,em_unit,t_unit,vel_unit,u_unit,
     &    amom_unit,sd_unit,rho_unit,p_unit,ent_unit

!
! Cosmology
!
      integer ired,ncdumps
      double precision redshift,h_0,omegam,omegal,th,ascale,z,
     &                  hnow,ainteg,cosbox,dtcosmax,cosm_h0,
     &                  redump(100)

      end module miscvars
