      module stubs
!****************************************************************
! This module is a component of VINE, copyright by the authors, *
! as listed below. Please see the COPYING and LICENSE files     *
! in the parent directory for additional information.           *
! This notification is present as an attempt to follow the      *
! recommendations for licensing published on the FSF page.      * 
!****************************************************************
!  This file contains the VINE module that contains stub        *
!  routines that are required for successfully compiling the    *
!  code without various modules included.                       *
!                                                               *
!                                                               *
!  A VINE module written by                                     *
!  Andy Nelson, Markus Wetzstein and Thorsten Naab              *
!                                                               *
!****************************************************************

      use dimens, only:idim,ndim,ndim1,ndim3

      implicit none
      public
      private idim,ndim,ndim1,ndim3
      private dummy

      contains
!
! Intel compiler wants at least one routine in every module,
! so make one here.
!
!-------------------------------------------------------------------
      subroutine dummy
      return
      end subroutine dummy
!-------------------------------------------------------------------
!
!
! Self Force (e.g. gravity) not used:
!
!
#ifndef HAVE_DIRSUM
!-------------------------------------------------------------------
      subroutine host_dirsum(npart,isoft,eps2,posmas,vel,acc,poten)
      integer, intent(in):: isoft,npart
      double precision, intent(in)::eps2
      double precision, intent(in):: posmas(ndim1,npart),vel(ndim,npart)
      double precision, intent(inout):: acc(ndim,npart), poten(npart)

      stop'dirsum:Host based direct summation forces are not installed'

      end subroutine host_dirsum

#endif
#ifndef SELFFORCE
!-------------------------------------------------------------------
      subroutine dirsum(igrape,npart,isoft,eps2,posmas,vel,acc,poten)
      integer, intent(in):: igrape,isoft,npart
      double precision, intent(in)::eps2
      double precision, intent(in):: posmas(ndim1,npart),vel(ndim,npart)
      double precision, intent(inout):: acc(ndim,npart), poten(npart)

      stop'dirsum:Self forces are not installed'

      end subroutine dirsum
!-------------------------------------------------------------------
      subroutine acceln2(npart,nact,ntot,listpt,isoft,eps2,apcorr,
     &                   acc,poten)
      integer, intent(in):: isoft,npart
      integer, intent(in):: nact,ntot,listpt(ntot)                        
      double precision, intent(in)::eps2
      double precision, intent(inout):: apcorr(0:ndim,nact)
      double precision, intent(inout):: acc(ndim,npart),poten(npart) 

      stop'acceln2:Self forces are not installed'

      end subroutine acceln2
!-------------------------------------------------------------------
      subroutine calcgravgroup(igrape,igrapelim,isoft,eps2,npart,iclump,
     &                         nact,ntot,listpt,na,nn,latoms,lnodes,
     &                         vel,apcorr,acc,poten)
      integer, intent(in):: igrape,igrapelim,isoft
      integer, intent(in):: npart,iclump
      integer, intent(in):: nact,ntot,listpt(ntot)
      integer, intent(inout):: na,latoms(idim), nn,lnodes(nn)
      double precision, intent(in):: eps2
      double precision, intent(in):: vel(ndim,npart)
      double precision, intent(inout):: apcorr(0:ndim,nact)
      double precision, intent(inout):: acc(ndim,npart), poten(npart)

      stop'calcgravgroup:Self forces are not installed'

      end subroutine calcgravgroup
!-------------------------------------------------------------------
      subroutine calcgrav(itreept,iact,nact,isoft,npart,eps2,
     &                    na0,na1,na,nn,latoms,lnodes,apcorr,acc,poten)
      integer, intent(in):: itreept,iact,nact,isoft,na0,na1,npart
      integer, intent(in):: na,latoms(na), nn,lnodes(nn)
      double precision, intent(in):: eps2
      double precision, intent(inout):: apcorr(0:ndim,nact)
      double precision, intent(inout):: acc(ndim,npart), poten(npart)

      stop'calcgrav:Self forces are not installed'

      end subroutine calcgrav
!-------------------------------------------------------------------
      subroutine revpole(igrape,irevise,imac,treeacc)
      integer, intent(in):: igrape,irevise,imac
      double precision, intent(in):: treeacc
      stop 'revpole:Self forces are not installed'
      end subroutine revpole
!-------------------------------------------------------------------
      subroutine initpole(npart)
      integer, intent(in):: npart
      stop 'initpole:Self forces are not installed'
      end subroutine initpole
!-------------------------------------------------------------------
#endif
!
!
! Grape hardware not used:
!
!
#ifndef GRAPE
!-------------------------------------------------------------------
      subroutine lock_grape

      print *,'Grapes have withered on the VINE'
      print *,'Nothing but sour grapes left so I''m '
      print *,'wrathfully sending lists to host instead'
      return
      end subroutine lock_grape 
!-------------------------------------------------------------------
      
      subroutine unlock_grape
      print *,'nothing happens'
      return
      end subroutine unlock_grape
!-------------------------------------------------------------------
#endif
!
!
! Periodic boundaries not used:
!
!
#ifndef PERIODIC
!-------------------------------------------------------------------
      subroutine init_ewald
      stop'Periodic boundaries not installed'
      return
      end subroutine init_ewald
!-------------------------------------------------------------------
      subroutine apply_ewald(npart,posmas,acc,poten)
      integer npart
      double precision posmas(ndim1,1),acc(ndim,1)
      double precision poten(1)
      stop'apply_ewald:periodic boundaries not installed'
      return
      end subroutine apply_ewald
!-------------------------------------------------------------------
      subroutine init_rand(ivalue)
      integer ivalue
      stop'no random number generator is known'
      return
      end subroutine init_rand
!-------------------------------------------------------------------
      subroutine shift_box_to_original(npart,npoim,posmas,posmaspm)
      integer npart,npoim
      double precision posmas(ndim1,1),posmaspm(ndim1,1)
      stop'Periodic boundaries not installed'
      return
      end subroutine shift_box_to_original
!-------------------------------------------------------------------
      subroutine shift_box_to_current(npart,npoim,posmas,posmaspm)
      integer npart,npoim
      double precision posmas(ndim1,1),posmaspm(ndim1,1)
      stop'Periodic boundaries not installed'
      return
      end subroutine shift_box_to_current
!-------------------------------------------------------------------
      subroutine boxshift(npart,npoim,posmas,posmaspm)
      integer npart,npoim
      double precision posmas(ndim1,1),posmaspm(ndim1,1)
      stop'Periodic boundaries not installed'
      return
      end subroutine boxshift
!-------------------------------------------------------------------
      subroutine checkboxlimits(npart,npoim,dumposmas,dumposmaspm)
      integer npart,npoim
      double precision dumposmas(ndim1,1),dumposmaspm(ndim1,1)
      stop'Periodic boundaries not installed'
      return
      end subroutine checkboxlimits
!-------------------------------------------------------------------
#endif

!
! Leapfrog integrator not used:
!
!
#ifndef LF
!-------------------------------------------------------------------
      subroutine lf_correct(nactive_s,nactive,nactivepm,
     &                      iactive_s,iactive,iactivepm,
     &                      itstepbin,itstepbinpm,dtincr,
     &                      deltat,deltatpm)
      integer, intent(in)::nactive_s   ,nactive   ,nactivepm
      integer, intent(in)::iactive_s(1),iactive(1),iactivepm(1)
      integer, intent(in)::itstepbin(1),itstepbinpm(1)
      double precision deltat(1),deltatpm(1)
      double precision dtincr
      stop'LF integrator not installed'
      end subroutine lf_correct
!-------------------------------------------------------------------
#endif
!
!
! Cosmology not used:
!
!

#ifndef COSMO
!-------------------------------------------------------------------
      subroutine cosmo_dumptime(n,tnextdump)
      integer, intent(out):: n
      double precision, intent(out):: tnextdump
      n=0
      tnextdump=0d0
      stop'cosmology flag not turned on at compilation time'
      return
      end subroutine cosmo_dumptime
!-------------------------------------------------------------------
      subroutine cosmic_time(rednext,tnextdump,omegam,omegal,cosm_h0)
      double precision rednext,tnextdump,omegam,omegal,cosm_h0
      stop'cosmology flag not turned on at compilation time'
      return
      end subroutine cosmic_time
!-------------------------------------------------------------------
      subroutine expansion(nactive,iactive,nactivepm,iactivepm,
     &                    npart,npoim,acc,vel,accpm,velpm)
      integer nactive  ,npart,iactive  (1)
      integer nactivepm,npoim,iactivepm(1)
      double precision acc  (ndim,1),vel  (ndim,1)
      double precision accpm(ndim,1),velpm(ndim,1)
      stop'cosmology flag not turned on at compilation time'

      return
      end subroutine expansion
!-------------------------------------------------------------------
      subroutine cosmo_ts(nactivef,nactivefpm,iactive,iactivepm,
     &                    itstepbin,itstepbinpm,dtincr,dtrat,dtratpm)
      integer nactivef,iactive(1),nactivefpm,iactivepm(1)
      integer itstepbin(1),itstepbinpm(1)
      double precision dtincr,dtrat(1),dtratpm(1)
      stop'cosmology flag not turned on at compilation time'
      return
      end subroutine cosmo_ts
!-------------------------------------------------------------------
      subroutine cosmo_vel(nactive,nactivepm,iactive,iactivepm,
     &                     npart,npoim,deltat,deltatpm,
     &                     acc1,vel,dumvel,acc1pm,velpm,dumvelpm)
      integer nactive,npart,npoim,nactivepm,iactive(1),iactivepm(1)
      double precision deltat(1),deltatpm(1)
      double precision acc1(ndim,1),acc1pm(ndim,1)
      double precision vel  (ndim,1),dumvel  (ndim,1)
      double precision velpm(ndim,1),dumvelpm(ndim,1)
      stop'cosmology flag not turned on at compilation time'
      return
      end subroutine cosmo_vel
!-------------------------------------------------------------------
      subroutine cosmic_stepa(ascale,dtincr,cosm_h0,omegam,omegal)
      double precision ascale,dtincr,cosm_h0,omegam,omegal
      stop'cosmology flag not turned on at compilation time'
      return
      end subroutine cosmic_stepa
!-------------------------------------------------------------------

#endif


!
!
! XDR libraries not used:
!
!
#ifndef XDR
!-------------------------------------------------------------------
      integer function initxdr(file,opt,logopt)
      logical logopt
      character(len=1) opt
      character(len=*) file
      initxdr=-1
      stop'inidxdr stub:xdr library not compiled in'
      return
      end function initxdr
!-------------------------------------------------------------------
      integer function ixdrdmat(id,ilen,darr)
      integer id,ilen
      double precision darr(ilen)
      ixdrdmat=-1
      stop'ixdrdmat stub:xdr library not compiled in'
      return
      end function ixdrdmat
!-------------------------------------------------------------------
      integer function ixdrimat(id,ilen,iarr)
      integer id,ilen,iarr(ilen)
      ixdrimat=-1
      stop'ixdrimat stub:xdr library not compiled in'
      return
      end function ixdrimat
!-------------------------------------------------------------------
      integer function ixdrrewind(id)
      integer id
      ixdrrewind=-1
      stop'ixdrrewind stub:xdr library not compiled in'
      return
      end function ixdrrewind
!-------------------------------------------------------------------
      integer function ixdrclose(id)
      integer id
      ixdrclose=-1
      stop'ixdrclose stub:xdr library not compiled in'
      return
      end function ixdrclose
!-------------------------------------------------------------------
#endif
!
!
! SPH not used:
!
!
#ifndef SPH
!-------------------------------------------------------------------
      subroutine hadjust
      print *,'No SPH particles because SPH is not compiled in'
      end subroutine hadjust
!-------------------------------------------------------------------
      subroutine SPHkerntablinit
      stop'Kernel Init: no kernel not found'
      return
      end subroutine SPHkerntablinit
!-------------------------------------------------------------------
      subroutine SPHkern(ichoice,nneigh,ntot,distmas,WdW)
      double precision distmas(ndim3,1,1),WdW(2,1,1)
      integer nneigh(1),ntot,ichoice
      stop'SPHkern stub: SPH preprocessor variable not set'
      return
      end subroutine SPHkern
!-------------------------------------------------------------------
      subroutine density(nactive_c,iactive_c,
     &          npart,npart_sph,posmas,vel,h,rho,divv)
      integer nactive_c,iactive_c(1)
      integer npart,npart_sph
      double precision posmas(ndim1,1),vel(ndim,1),h(1)
      double precision rho(1),divv(1)
      stop'density stub: SPH preprocessor variable not set'
      return
      end subroutine density
!-------------------------------------------------------------------
      subroutine sphaccel(nactive_c,iactive_c,npart,npart_sph,
     &                     vel,rho,alpha,acc)
      integer nactive_c,iactive_c(1),npart,npart_sph
      double precision vel(ndim,1),rho(1),alpha(1),acc(ndim,1)
      stop'sphaccel stub: SPH preprocessor variable not set'
      return
      end subroutine sphaccel
!-------------------------------------------------------------------
      subroutine hdot(nactive_s,iactive_s,nactive_n,iactive_n,
     &      itstepbin,npart,npart_sph,dtincr,h,divv,dhdt)
      integer nactive_s,iactive_s(1),nactive_n,iactive_n(1)
      integer itstepbin(1),npart,npart_sph
      double precision dtincr, rho(1),divv(1),h(1),dhdt(1)
      stop'hdot stub: SPH preprocessor variable not set'
      return
      end subroutine hdot
!-------------------------------------------------------------------
      subroutine viscdot(nactive_s,iactive_s,npart,npart_sph,
     &                               h,alpha,divv,alphadot)
      integer nactive_s,iactive_s(1),npart,npart_sph
      double precision h(1),alpha(1),divv(1),alphadot(1)
      stop'viscdot stub: SPH preprocessor variable not set'
      return
      end subroutine viscdot
!-------------------------------------------------------------------
      subroutine cour_ts(nactive_s,iactive_s,itstepbin,dtincr,
     &                       h,alpha, dtrat)
      integer nactive_s,iactive_s(1),itstepbin(1)
      double precision dtincr,h(1),alpha(1),dtrat(1)
      if(nactive_s.eq.0)return
      stop'cour_ts stub: SPH preprocessor variable not set'
      return
      end subroutine cour_ts
!-------------------------------------------------------------------
      subroutine dchange(nactive_s,iactive_s,itstepbin,dtincr,
     &                                rho,divv,dtrat)
      integer nactive_s,iactive_s(1),itstepbin(1)
      double precision dtincr,rho(1),divv(1),dtrat(1)
      stop'dchange stub: SPH preprocessor variable not set'
      return
      end subroutine dchange
!-------------------------------------------------------------------
      subroutine stable(nactivef_s,iactive_s,npart,npart_sph,posmas,rho)
      integer nactivef_s,iactive_s(1),npart,npart_sph
      double precision posmas(ndim1,1),rho(1)
      stop'stable stub: SPH preprocessor variable not set'
      return
      end subroutine stable
!-------------------------------------------------------------------
      subroutine udot(nactive_s,iactive_s,npart_sph,npoim,posmas,
     &                rho,u,posmaspm,dudt)
      integer nactive_s,iactive_s(1),npart,npart_sph,npoim
      double precision rho(1),u(1),dudt(1)
      double precision posmaspm(ndim1,1),posmas(ndim1,1)
      stop'udot stub: SPH preprocessor variable not set'
      return
      end subroutine udot
!-------------------------------------------------------------------
      subroutine eos(npart,npart_sph,npoim,rho,material,
     &               posmas,posmaspm,u)
      integer npart,npart_sph,npoim
      integer material(1)
      double precision rho(1),u(1)
      double precision posmas(ndim1,1),posmaspm(ndim1,1)
      stop'eos stub: SPH preprocessor variable not set'
      return
      end subroutine eos
!-------------------------------------------------------------------
      subroutine mktable(npoim,posmaspm,idustev,colfrac)
      integer, intent(in):: npoim,idustev
      double precision, intent(in):: posmaspm(ndim1,npoim)
      double precision, intent(in):: colfrac
      end subroutine mktable
!-------------------------------------------------------------------
      subroutine sedsynth(nana_step,npart_sph,npoim,posmas,rho,u,
     &                    posmaspm,time)
      integer, intent(in):: nana_step,npart_sph,npoim
      double precision, intent(in):: time
      double precision, intent(in):: rho(npart_sph),u(npart_sph)
      double precision, intent(in):: posmas  (ndim1,npart_sph)
      double precision, intent(in):: posmaspm(ndim1,npoim)
      end subroutine sedsynth
!-------------------------------------------------------------------
#endif
!
!
! point masses not used:
!
!
#ifndef PM
!-------------------------------------------------------------------
      subroutine extrap_halfpm(deltatpm)
      double precision deltatpm(1)
      return
      end subroutine extrap_halfpm
!-------------------------------------------------------------------
      subroutine halfupdatepm(nactivefpm,nactivepm,iactivepm,
     &                     itstepbinpm,dtincr,deltatpm)
      integer nactivefpm,nactivepm,itstepbinpm(1),iactivepm(1)
      double precision dtincr,deltatpm(1)
      return
      end subroutine halfupdatepm
!-------------------------------------------------------------------
      subroutine fullupdatepm(nactivepm,iactivepm,deltatpm)
      integer nactivepm, iactivepm(1)
      double precision deltatpm(1)
      return
      end subroutine fullupdatepm
!-------------------------------------------------------------------
      subroutine pmerrors(i12step,nactivepm,iactivepm,
     &                 itstepbinpm,dtincr,dtratpm)
      integer i12step,nactivepm,itstepbinpm(1),iactivepm(1)
      double precision dtincr, dtratpm(1)
      return
      end subroutine pmerrors
!-------------------------------------------------------------------
      subroutine lf_correctpm(nactivepm,iactivepm,dtincr,itstepbinpm,
     &     deltatpm)
      integer nactivepm,itstepbinpm(1),iactivepm(1)
      double precision dtincr,deltatpm(1)
      return
      end subroutine lf_correctpm
!-------------------------------------------------------------------
      subroutine copybackpm(nactivepm,iactivepm)
      integer nactivepm,iactivepm(1)
      return
      end subroutine copybackpm
!-------------------------------------------------------------------
      subroutine pmgrav(nactive,iactive,npart,nactivepm,iactivepm,npoim,
     &           posmas,acc,poten,posmaspm,hpm,accpm,potpm)
      integer npoim,npart
      integer nactive, nactivepm,iactive(1), iactivepm(1)
      double precision posmas  (ndim1,1), acc  (ndim,1)
      double precision poten(1)
      double precision posmaspm(ndim1,1), accpm(ndim,1)
      double precision potpm(1), hpm(1)
      stop'pmgrav stub: PM preprocessor variable not set'
      return
      end subroutine pmgrav
!-------------------------------------------------------------------
      subroutine remove(i12step,nremove)
      integer, intent(in):: i12step, nremove
      stop'remove stub: PM preprocessor variable not set'
      return
      end subroutine remove
!-------------------------------------------------------------------

#else
!PM is defined

#ifndef LF
!-------------------------------------------------------------------
      subroutine lf_correctpm(nactivepm,iactivepm,dtincr,itstepbinpm,
     $     deltatpm)
      integer nactivepm,itstepbinpm(1),iactivepm(1)
      double precision dtincr,deltatpm(1)
      return
      end subroutine lf_correctpm
!-------------------------------------------------------------------

#endif
#endif
      end module stubs
