
pro vineread,pos,vel,mass,h,npart,ngas,nstep,tkin,tgrav,tterm,u,rho,t,gamma,ipindx,alpha,dtmax,deltnew,tbins,poten,ascale,ndumps,basename,path,iheader,dheader

;====================================================================
; This IDL routine reads in a set of VINE dump files from disk
; This version does NOT read in the point mass data at the end of the
; file, but it is easy to add if you should need it.
;
; Note that currently, the material array in the dump file is not
; passed out to IDL, as it is not used in the public VINE version. If
; you want to add it here, define a material array similar to the
; other longint arrays and fill it with x14.
;
; Written by Markus Wetzstein
;====================================================================

; user settable options
bigendian=1

ndumps=10
firstdump='0000'

basename='MY_FILE_'

path_ext=''

path='/mypath/here/'

;=====================================================================
; code to read in a set of dump files


path=path+path_ext

close,/all

; get header and number of particles from file

fnams=basename+firstdump
if(bigendian)then begin
    openr,fileunit,path+fnams,/f77,/get_lun,/swap_if_little_endian
endif else begin
    openr,fileunit,path+fnams,/f77,/get_lun
endelse
headlength=0l
readu,fileunit,headlength
free_lun,fileunit

iheader=lonarr(headlength)
dheader=dblarr(headlength)
if(bigendian)then begin
    openr,fileunit,path+fnams,/f77,/get_lun,/swap_if_little_endian
endif else begin
    openr,fileunit,path+fnams,/f77,/get_lun
endelse
readu,fileunit,iheader,dheader

free_lun,fileunit
idim=iheader(1)
idimsph=iheader(2)

;now def variables


x=dblarr(idim)
x1=x
x2=dblarr(4,idim)
x3=dblarr(3,idim)
x4=x
x5=x
x6=x
x7=x
x7a=x
x9=x
x11=lonarr(idim)
x13=x11
x14=x11
if(idimsph ne 0)then begin
    x8=dblarr(idimsph)
    x10=dblarr(idimsph)   ;rhotemplate
    x12=dblarr(idimsph)
endif
h=dblarr(idim,ndumps)
npart=lonarr(ndumps)
ngas=lonarr(ndumps)
nstep=lonarr(ndumps)
t=dblarr(ndumps)
dtmax=dblarr(ndumps)
deltnew=dblarr(ndumps)
tbins=lonarr(idim,ndumps)
gamma=dblarr(ndumps)
tkin=dblarr(ndumps)
tgrav=dblarr(ndumps)
tterm=dblarr(ndumps)
ascale=dblarr(ndumps)
pos=dblarr(3,idim,ndumps)
vel=dblarr(3,idim,ndumps)
mass=dblarr(idim,ndumps)
poten=dblarr(idim,ndumps)
ipindx=lonarr(idim,ndumps)
if(idimsph ne 0)then begin
    rho=dblarr(idimsph,ndumps)
    u=dblarr(idimsph,ndumps)
    alpha=dblarr(idimsph,ndumps)
endif


j=0
fnam=firstdump

For k=0,ndumps-1 DO BEGIN

fnams=basename+fnam

print,'Reading dump ', fnams
if(bigendian)then begin
    openr,fileunit,path+fnams,/f77,/get_lun,/swap_if_little_endian
endif else begin
    openr,fileunit,path+fnams,/f77,/get_lun
endelse
;read header version dump file 
if(idimsph ne 0)then begin
    print,'with sph'
    readu,fileunit,iheader,dheader,x2,x3,x1,x8,x10,x12,x7a,x11,x14,x13
endif else begin
    print,'without sph'
    readu,fileunit,iheader,dheader,x2,x3,x1,x7a,x11,x14,x13
endelse

    print,iheader(1:3),dheader(0)

    npart(k)=iheader(1)
    ngas(k)=iheader(2)
    nstep(k)=iheader(3)

    t(k)=dheader(0)
    dtmax(k)=dheader(1)
    deltnew(k)=dheader(2)
    
    gamma(k)=dheader(3)
    tkin(k)=dheader(4)
    tgrav(k)=dheader(5)
    tterm(k)=dheader(6)

    ascale(k)=dheader(7)
    cosbox=dheader(8)

;convert fortran indices to idl indices 
    x11=lindgen(npart(k))
;restore original order of particles
    idx=lindgen(npart(k))
    if(idimsph ne 0)then begin    
        idx2=lindgen(ngas(k))
    endif
    h(x11(idx),k)=x1

    pos(0,x11(idx),k)=x2(0,*)
    pos(1,x11(idx),k)=x2(1,*)
    pos(2,x11(idx),k)=x2(2,*)
    mass(x11(idx),k) =x2(3,*)

    vel(0,x11(idx),k)=x3(0,*)
    vel(1,x11(idx),k)=x3(1,*)
    vel(2,x11(idx),k)=x3(2,*)

    poten(x11(idx),k)=x7a
    ipindx(*,k)=x11
    if(idimsph ne 0)then begin
        rho(x11(idx2),k)=x10
        alpha(x11(idx2),k)=x12
        u(x11(idx2),k)=x8
    endif else begin
        rho=0d0
        u=0d0
        alpha=0d0
    endelse
    tbins(x11(idx),k)=x13
;close file
    free_lun,fileunit

;find new dump file extension
    fnam=fnam+1

    IF (fnam lt 10) THEN BEGIN
        fnam=string(format='(i1)',fnam)
        fnam='000'+fnam
    ENDIF
    IF ((fnam lt 100) and (fnam gt 9)) THEN BEGIN
        fnam=string(format='(i2)',fnam)
        fnam='00'+fnam
    ENDIF

    IF ((fnam lt 1000) and (fnam gt 99)) THEN BEGIN
        fnam=string(format='(i3)',fnam)
        fnam='0'+fnam
    ENDIF

ENDFOR

return
end

